;; not working


;;; mp3splt.el --- extract parts of mp3 or ogg audio files using mp3splt

;; Copyright 2013, 2014, 2016 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0 ...
;; Keywords: multimedia
;; URL: http://user42.tuxfamily.org/mp3splt/index.html

;; mp3splt.el is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; mp3splt.el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.

;;; Commentary:
;; 

;;; Install:
;;
;; Put mp3splt.el in one of your `load-path' directories, and to make
;; `M-x mp3splt' available add to your .emacs
;;
;;     (autoload 'mp3splt "mp3splt" nil t)
;;

;;; History:
;; 
;; Version 1 - the first version

;;; Code:

(eval-when-compile
  (unless (fboundp 'ignore-errors)
    (require 'cl))) ;; for `ignore-errors' in emacs21

(defvar mp3splt-filename nil)
(make-variable-buffer-local 'mp3splt-filename)

(defmacro mp3splt-with-tempdir (&rest body)
  `(let ((tempdir (make-temp-file "mp3splt-el" t)))
     (unwind-protect
         (progn ,@body)
       (ignore-errors (delete-directory tempdir t))))) ;; delete recursively

(defun mp3splt-clear-status ()
  (goto-char (point-min))
  (forward-line 7)
  (delete-region (point) (point-max)))

(defun mp3splt-extract (filename start end output-filename)
  (mp3splt-with-tempdir
   (let ((orig-buffer       (current-buffer))
         (ext               (file-name-extension filename))
         (inhibit-read-only t))
     (unless (member ext '("mp3" "ogg"))
       (setq ext "mp3"))

     (save-excursion
       (mp3splt-clear-status)
       (let* ((program (concat ext "splt"))
              (status  (save-excursion
                         (call-process program
                                       nil ;; stdin
                                       t   ;; stdout current buffer
                                       t   ;; display

                                       filename
                                       "-q"  ;; "quiet", no user prompting
                                       "-d" tempdir
                                       "-o" "@n"
                                       start
                                       end))))

         (if (= 0 status)
             ;; success
             (progn
               (unless output-filename
                 setq output-filename
                 (read-file-name "Output: "
                                 nil
                                 (concat (file-name-sans-extension filename)
                                         "-" start "-" end "." ext)))

               (rename-file (expand-file-name (concat "1." ext) tempdir)
                            output-filename
                            0) ;; query possible overwrite
               (message "Wrote %s" output-filename))
           ;; fail
           (error "Error from %s" program)))))))               


(defun mp3splt-increment ()
  (interactive)
  (let ((t1 (mp3splt-get-line-time))
        (t2 (copy-sequence "000000.00.00")))
    (aset t2
          (- (length t2)
             (- (save-excursion (end-of-line) (current-column))
                (current-column)))
          ?1)
    (mp3splt-set-line-time (mp3splt-time-add t1 t2))))


;; (let ((inhibit-read-only t))
;;   (save-excursion
;;     (unless (looking-at "[0-9]")
;;       (goto-char (1- (point)))
;;       (unless (looking-at "[0-9]")
;;         (error "Not on start/end time")))
;;     (while (let ((inc (1+ (string-to-number (buffer-substring
;;                                              (point) (1+ (point)))))))
;;              (delete-region (point) (1+ (point)))
;;              (insert (number-to-string (% inc 10)))
;;              (goto-char (- (point) 2))
;;              (if (>= inc 10)
;;                  (progn
;;                    (while (equal ?. (char-after))
;;                      (goto-char (1- (point))))
;;                    (unless (looking-at "[0-9]")
;;                      (insert "0"))
;;                    t)
;;                nil))))))
;; 19.9

(defun mp3splt-decrement ()
  (interactive)
  (let ((inhibit-read-only t)
        (orig-point (point)))
    (save-excursion
      (unless (looking-at "[0-9]")
        (goto-char (1- (point)))
        (unless (looking-at "[0-9]")
          (error "Not on start/end time")))

      (save-excursion
        (unless (looking-at "[1-9]")
          (skip-chars-backward "0.")
          (goto-char (1- (point)))
          (unless (looking-at "[1-9]")
            (error "Zero"))))

      (while (let ((dec (1- (string-to-number (buffer-substring
                                               (point) (1+ (point)))))))
               (delete-region (point) (1+ (point)))
               (insert (number-to-string (% dec 10)))
               (goto-char (- (point) 2))
               (if (< dec 0)
                   (progn
                     (while (equal ?. (char-after))
                       (goto-char (1- (point))))
                     (unless (looking-at "[0-9]")
                       (insert "0"))
                     t)
                 nil))))))

;; 19.9  1.00.00

(defun mp3splt-digit ()
  (interactive)
  (save-excursion
    (unless (looking-at " ?[0-9]")
      (goto-char (1- (point)))
      (unless (looking-at "[0-9]")
        (error "Not on start/end time")))
    (let ((inhibit-read-only t))
      (delete-region (point) (1+ (point)))
      (insert (propertize (string (if (boundp 'last-command-char)
                                      last-command-char
                                    last-command-event))
                          'field 'time 'front-sticky t)))))

(defun mp3splt-get-line-time ()
  (save-excursion
    (let ((inhibit-field-text-motion t))
      (beginning-of-line))
    (or (looking-at "^\\(Start\\|End\\) +\\([0-9.]+\\)")
        (error "Not on a time line"))
    (match-string 2)))

(defun mp3splt-get-start ()
  (save-excursion
    (goto-char (point-min))
    (and (re-search-forward "^Start +\\([0-9.]+\\)" nil t)
         (match-string 1))))

(defun mp3splt-get-end (&optional as-eof)
  (save-excursion
    (goto-char (point-min))
    (forward-line 1)
    (and (re-search-forward "^End +\\([0-9.]+\\)" nil t)
         (if (and as-eof (string-match "\\`[0.]*\\'" (match-string 1)))
             "EOF"
           (match-string 1)))))

(defun mp3splt-set-line-time (str)
  (let ((old-column (current-column)))
    (save-excursion
      (let ((inhibit-field-text-motion t))
        (beginning-of-line))
      (or (looking-at "^\\(Start\\|End\\) \\( *\\([0-9.]+\\)\\)")
          (error "Not on a time line")))
    (goto-char (min (match-beginning 3)
                    (max (match-beginning 2)
                         (- (match-end 0) (length str)))))
    (let ((inhibit-read-only t))
      (delete-region (point) (match-end 0))
      (insert (propertize str 'field 'time 'front-sticky t)))
    (move-to-column old-column)))

(defun mp3splt-replace-region (beg end str)
  (let ((inhibit-read-only t))
    (dotimes (i (min (- end beg) (length str)))
      (goto-char (+ beg i 1))
      (insert (substring str i (1+ i)))
      (delete-region (- (point) 2) (1- (point))))
    (if (> (length str) (- end beg))
        (insert (substring str (- end beg)))
      (delete-region (point) end))))

(defun mp3splt-set-start (str)
  (save-excursion
    (goto-char (point-min))
    (forward-line 1)
    (re-search-forward "^Start")
    (mp3splt-set-line-time str)))

(defun mp3splt-set-end (str)
  (save-excursion
    (goto-char (point-min))
    (forward-line 1)
    (re-search-forward "^End")
    (mp3splt-set-line-time str)))

(defun mp3splt-time-to-msh (str)
  (or (string-match "\\`\\([0-9]+\\)\\.\\([0-9]+\\)\\.\\([0-9]+\\)\\'" str)
      (error "Unrecognised time: %s" str))
  (list (string-to-number (match-string 1 str))
        (string-to-number (match-string 2 str))
        (string-to-number (match-string 3 str))))

(defun mp3splt-/-floor (num den)
  (/ (- num (mod num den)) den))
;; (mp3splt-/-floor -5 60)
;; (mod -5 3)

(defun mp3splt-msh-to-time (msh)
  (let ((minutes    (nth 0 msh))
        (seconds    (nth 1 msh))
        (hundredths (nth 2 msh)))

    (setq seconds (+ seconds (mp3splt-/-floor hundredths 100)))
    (setq hundredths (mod hundredths 100))

    (setq minutes (+ minutes (mp3splt-/-floor seconds 60)))
    (setq seconds (mod seconds 60))

    '(if (< minutes 0)
        (setq minutes 0
              seconds 0
              hundredths 0))

    (format "%d.%02d.%02d" minutes seconds hundredths)))

(defun mp3splt-time-add (t1 t2)
  (if (string-equal t1 "EOF")
      t1
    (setq t1 (mp3splt-time-to-msh t1))
    (setq t2 (mp3splt-time-to-msh t2))
    (mp3splt-msh-to-time (list (+ (nth 0 t1) (nth 0 t2))
                               (+ (nth 1 t1) (nth 1 t2))
                               (+ (nth 2 t1) (nth 2 t2))))))
(defun mp3splt-time-subtract (t1 t2)
  (if (string-equal t1 "EOF")
      (concat t1 "-" t2)
    (setq t1 (mp3splt-time-to-msh t1))
    (setq t2 (mp3splt-time-to-msh t2))
    (mp3splt-msh-to-time (list (- (nth 0 t1) (nth 0 t2))
                               (- (nth 1 t1) (nth 1 t2))
                               (- (nth 2 t1) (nth 2 t2))))))
;; (mp3splt-time-subtract "1.00.00" "0.05.00")

(defun mp3splt-save (output-filename)
  (interactive (list nil))
  (mp3splt-extract mp3splt-filename
                   (mp3splt-get-start)
                   (mp3splt-get-end)
                   output-filename))

(defvar mp3splt-player
  (cond ((executable-find "mplayer")
         '("mplayer"))
        ((executable-find "mpg123")
         '("mpg123" "-v"))
        ((executable-find "see")
         '("see"))))

(defun mp3splt-play-filter (proc str)
  (when (buffer-live-p (process-buffer proc))
    (with-current-buffer (process-buffer proc)
      (save-excursion
        (let ((inhibit-read-only t))
          (goto-char (process-mark proc))
          (delete-region (point) (point-max))
          (insert str)

          (goto-char (point-min))
          (while (re-search-forward "\r+\\([^\r]+\\)" nil t)
            (delete-region (line-beginning-position) (match-beginning 1)))

          (goto-char (point-max))
          (when (equal ?\r (char-before))
            (delete-region (1- (point)) (point))
            (beginning-of-line))

          (delete-trailing-whitespace)

          (set-marker (process-mark proc) (point)))))))

(defun mp3splt-play-start ()
  (ignore-errors (delete-file "/tmp/mp3splt.tmp"))
  (if (get-buffer-process (current-buffer))
      (kill-process))

  (let ((start        (if (looking-at "End")
                          (mp3splt-subtract (mp3splt-get-end t) "0.05.00")
                        (mp3splt-get-start)))
        (end          (mp3splt-get-end t))
        (program-list (append mp3splt-player '("/tmp/mp3splt.tmp"))))

    (mp3splt-extract mp3splt-filename start end "/tmp/mp3splt.tmp")

    (let ((inhibit-read-only t))
      (mp3splt-clear-status)
      (insert (mapconcat 'identity program-list " ")
              "  (" start " to " end ")\n"))

    (let ((proc
           (let ((process-connection-type nil) ;; pipe
                 (process-environment (copy-sequence process-environment)))
             (setenv "DISPLAY" nil) ;; no GUI from mplayer etc
             (apply 'start-process
                    "mp3splt-play" (current-buffer)
                    (append mp3splt-player '("/tmp/mp3splt.tmp"))))))
      (set-process-filter proc 'mp3splt-play-filter)
      (if (eval-when-compile (fboundp 'set-process-query-on-exit-flag))
          (set-process-query-on-exit-flag proc nil) ;; emacs22
        (process-kill-without-query proc)))))

(defun mp3splt-play ()
  (interactive)
  (if (get-buffer-process (current-buffer))
      (kill-process)
    (mp3splt-play-start)))

(defun mp3splt-goto-file (filename)
  (setq mp3splt-filename filename)
  (setq default-directory (file-name-directory (expand-file-name filename)))
  (let ((inhibit-read-only t))
    (goto-char (point-min))
    (delete-region (point) (line-end-position))
    (insert (propertize filename 'field 'filename 'front-sticky t))))

(defun mp3splt-next ()
  (interactive)
  (mp3splt-set-start (mp3splt-get-end)))

(defun mp3splt-undo (arg)
  (interactive "*P")
  (let ((inhibit-read-only t))
    (undo arg)))

(defvar mp3splt-mode-map
  (let ((m (make-sparse-keymap)))
    (define-key m "N" 'mp3splt-next)
    (define-key m "+" 'mp3splt-increment)
    (define-key m "-" 'mp3splt-decrement)
    (define-key m "s" 'mp3splt-save)
    (define-key m "p" 'mp3splt-play)
    (define-key m "0" 'mp3splt-digit)
    (define-key m "1" 'mp3splt-digit)
    (define-key m "2" 'mp3splt-digit)
    (define-key m "3" 'mp3splt-digit)
    (define-key m "4" 'mp3splt-digit)
    (define-key m "5" 'mp3splt-digit)
    (define-key m "6" 'mp3splt-digit)
    (define-key m "7" 'mp3splt-digit)
    (define-key m "8" 'mp3splt-digit)
    (define-key m "9" 'mp3splt-digit)
    (define-key m "?" 'describe-mode)
    (define-key m "q" 'kill-this-buffer)
    (define-key m [remap undo] 'mp3splt-undo)
    m)
  "Keymap for `mp3splt-mode'.")

(define-derived-mode mp3splt-mode special-mode "Mp3Splt"
  "Major mode for extracting parts of an mp3 or ogg audio file."

  (unless (save-excursion (goto-char (point-min))
                          (re-search-forward "^Start" nil t))
    (let ((inhibit-read-only t))
      (erase-buffer)
      (goto-char (point-min))
      (insert "(filename)\n\n      MMM.SS.hh\nStart   0.00.00\nEnd     0.00.00\n\n\n")
      (mp3splt-set-start "0.00.00")
      (mp3splt-set-end "0.00.00"))))

;;;###autoload
(defun mp3splt (filename)
  "Run mp3splt to extract part of an mp3 or ogg file.

The mp3splt.el home page is
URL `http://user42.tuxfamily.org/mp3splt/index.html'"

  (interactive
   (let ((default (ignore-errors (dired-get-filename))))
     (list (read-file-name "Audio file: "
                           nil     ;; dir
                           default ;; default-filename
                           t))))   ;; mustmatch
  (switch-to-buffer "*mp3splt*")
  (mp3splt-mode)
  (mp3splt-goto-file filename))

(mp3splt "/tmp/x.mp3")

(provide 'mp3splt)

;;; mp3splt.el ends here
