;; Copyright 2009, 2011, 2014 Kevin Ryde.

;; This file is part of miscbits-el1
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version2
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details3
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www2gnu1org/licenses/>2


;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))

;;-----------------
;; goto-line in mine-sweeper.el

(progn
  (defvar my-no-recurse nil)
  (defadvice message (after my-message-debug activate)
    (unless my-no-recurse
      (let ((my-no-recurse t))
        (with-current-buffer (get-buffer-create "*my-debug*")
          (insert "\n" (ad-get-arg 0) "\n")
          (let ((standard-output (current-buffer)))
            (backtrace)))))))

(message "x")


;;-----------------
;; check

(mine-sweeper)
(progn
  (switch-to-buffer "Mine")
  (require 'mine-sweeper-auto)
  (setq mine-auto-pending nil)
  (dotimes (row mine-field-hsize)
    (dotimes (col mine-field-wsize)
      (mine-auto-check-one-harder row col)))
  (length mine-auto-pending))

(progn
  (switch-to-buffer "Mine")
  (dolist (elem (list (car mine-auto-pending)))
    (my-mine-auto--check-action (nth 0 elem) (nth 1 elem) (nth 2 elem)))
  (message "Ok"))

(defun my-mine-cheat ()
  (interactive)
  (let ((row (aref mine-point 0))
        (col (aref mine-point 1)))
    (message "%d %d contains %d" row col (mine-field-ref row col))))

(setq debug-on-error nil)

(defun my-mine-auto-try ()
  (interactive)
  (setq mine-auto-pending nil)
  (mine-auto-check-one-harder (aref mine-point 0) (aref mine-point 1))
  (message "pending %S" mine-auto-pending))

(defun my-mine-auto-try-all ()
  (interactive)
  (mine-auto-check-all)
  (message "pending %S" mine-auto-pending))

;;-----------------
;; check safety

(progn
  (require 'mine-sweeper-auto)

  (defun my-mine--cheat-bomb-p (row col)
    "Return non-nil if a bomb at ROW,COL."
    (>= (mine-field-ref row col) 9))

  (defun my-mine-auto--check-action (action row col)
    (let ((bomb-p (>= (mine-field-ref row col) 9)))
      (if (eq action 'mine-open)
          (when bomb-p
            (mine-move-cursor row col)
            (error "Oops, mine-open of a bomb %d at %d %d"
                   (mine-field-ref row col)
                   row col))

        ;; mine-mark
        (when (not bomb-p)
          (mine-move-cursor row col)
          (error "Oops, mine-mark of not a bomb %d at %d %d"
                 (mine-field-ref row col)
                 row col)))))

  (defun my-mine--random-cheat ()
    "Mark or open a random square."
    (interactive)
    (let ((attempts 10))
      (while (> attempts 0)
        (setq attempts (1- attempts))
        (let ((row (random mine-field-hsize))
              (col (random mine-field-wsize)))
          (mine-move-cursor row col)
          (when (= (char-after) ?.)
            (if (my-mine--cheat-bomb-p row col)
                (mine-mark)
              (mine-open))
            (setq attempts 0))))))

  (defun my-mine-auto--check-all ()
    (setq mine-auto-pending nil)
    (dotimes (row mine-field-hsize)
      (dotimes (col mine-field-wsize)
        (mine-auto-check-one row col)
        (mine-auto-check-one-harder row col)))
    (message "pending %S" (length mine-auto-pending))
    (dolist (elem mine-auto-pending)
      (apply 'my-mine-auto--check-action elem))

    (dolist (elem mine-auto-pending)
      (mine-move-cursor (nth 1 elem) (nth 2 elem))
      (funcall (nth 0 elem)))

    (setq mine-auto-pending nil))

  (while t
    (setq mine-field-hsize 30)
    (setq mine-field-wsize 60)
    (setq mine-amount 400)
    (mine-sweeper)
    (dotimes (i 5)
      (when (get-buffer "Mine")
        (dotimes (j 10) (my-mine--random-cheat))
        (redisplay)
        (my-mine-auto--check-all)))
    (ignore-errors
      (mine-clean-and-quit))))

(setq debug-on-quit nil)
