;; Copyright 2014, 2015, 2019, 2020 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------


;; has to be a plain string, not an expression say
(defgroup my-dummy-group nil
  "Dummy Group."
  :prefix "my-dummy-group-"
  :group 'external
  :link  '(url-link :tag "My Link"
                    (concat "file://" "/var/www/index.html")))
(customize-group 'my-dummy-group)

(setq my-var "abc")
(if (boundp 'my-var) my-var "default")

;;-----------------------------------------------------------------------------
;; delete old .internal.html

;; for i in *.internal.html; do b=`basename $i .html`; if [ -e $b.txt ]; then echo $i; fi done



;;-----------------
;; oeis-grep

;; with spaces
(oeis-grep "12,34,     56, 78")


;;-----------------
(require 'url)
url-hexify-string


;;-----------------------------------------------------------------------------
;; oeis-decimal-digits-at-point

(looking-at "\\<[0-9]*\\.")
".123

;;-----------------------------------------------------------------------------
(load (expand-file-name "~/lisp/oeis.el"))
oeis-number-list-at-point

+1,-2,+3,-4
1,2,
3,4

;; 1,2,
;; 4,5

1.5 123 456

x 1.5 y

(oeis--replace-regexp-in-string "\\`ab" "" "ababab")
(oeis--replace-regexp-in-string "\\`[0-9]+[^0-9]*" "" "5 123,456")
(oeis--replace-regexp-in-string "\\`[0-9]+[^0-9]*" "" "5 123,456")
replace-in-string

;;-----------------------------------------------------------------------------
(load (expand-file-name "../oeis.el"))
1489,1489,13489,1894
8,13,21,34,55,89,
oeis-grep
oeis-search
(oeis-superseeker-mail)
(oeis-ffap-enable)
oeis-browse-url
2*A021739
b000290.txt
b9999999.txt
a000027.txt
a9999999.txt

(setq debug-on-error t)
(require 'un-define) ;; xemacs utf-8
(oeis-superseeker-record-time)
(oeis-superseeker-check-time)

;;-----------------
time-since


;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))

;;------------------------------------------------------------------------------

;; (defcustom oeis-wget-program
;;   ;; automatically `risky-local-variable-p' due to name `-program'
;;   "wget"
;;   "wget program name"
;;   :type  'string
;;   :group 'oeis)


(oeis-anum-to-remote-url "A000033" 'internal-text)
(oeis-anum-to-url "A030303")
(oeis-anum-to-url "A021739")
(oeis-anum-to-name-from-internal-html "A003987")
(oeis-anum-to-name-from-names "A000002")
(oeis-anum-to-name "A999999")


;;------------------------------------------------------------------------------
;; search

http://oeis.org/search?fmt=short&q=bloodshed
http://oeis.org/search?fmt=data&q=bloodshed


;;------------------------------------------------------------------------------
;; tooltip by run-with-idle-timer

(oeis-tooltip-mode)
(find-file "~/OEIS/stripped")
;; A000290
;; A000291
;; A000291
;; A000294
;; A000295
(overlays-at (point))
(overlays-in (point-min) (point-max))
(setq oeis-tooltip-idle-timer nil)
(oeis-tooltip-start-timer)
(oeis-tooltip-idle-handler)
(flyspell-mode 0)

(defun noop () 
  (interactive)
  nil)

;;------------------------------------------------------------------------------
;; tooltip-hook called always in xemacs

(tooltip-mode 1)
(progn
  (defvar my-tooltip-hook 0)
  (defun my-tooltip-hook (event)
    (message "my-tooltip-hook %S %S" event (incf my-tooltip-hook))
    nil)
  (add-hook 'tooltip-hook 'my-tooltip-hook)
  )
(remove-hook 'tooltip-hook 'my-tooltip-hook)

(defun my-help-echo-handler-1 (&rest args)
  (message "help one %S %S" (length args) args)
  nil)
(overlay-put (make-overlay (point-min) (point-max)
                           (current-buffer) nil nil)
             'help-echo 'my-help-echo-handler-1)

;;------------------------------------------------------------------------------
;; oeis-search

(setq oeis-search-format "data")
(oeis-search "6, 16, 36, 72, 136, 256")
(oeis-search "1.6180339887498948482045868")
(oeis-grep "1.6180339887498948482045868")
123.000001
.000001

;;------------------------------------------------------------------------------
;; oeis-anum-fixup

(oeis-anum-fixup "A1")
(oeis-anum-fixup "A001")
(oeis-anum-fixup "A000001")
(oeis-anum-fixup "A00000001")
(oeis-anum-fixup "A123456")
(oeis-anum-fixup "A00123456")
(oeis-anum-fixup "A00023456")
(oeis-anum-fixup "A1234567")
(oeis-anum-fixup "A1234567890")



;;------------------------------------------------------------------------------

(let ((found (make-list 20 "")))
  (with-current-buffer (find-file-noselect "~/OEIS/names")
    (save-excursion
      (goto-char (point-min))
      (let ((case-fold-search nil))
        (while (re-search-forward "\\b[A-Za-z][a-z]*\\b" nil t)
          (and (>= (- (match-end 0) (match-beginning 0)) (length (car found)))
               (not (member (match-string 0) found))
               (progn
                 (setq found (cons (match-string 0) found))
                 (setcdr (nthcdr 20 found) nil)))))))
  found)
    

;;------------------------------------------------------------------------------
;; fontification-functions


  


;;------------------------------------------------------------------------------
;; two overlays

(defun my-help-echo-handler-1 (window overlay pos)
  (message "help one")
  nil)
(defun my-help-echo-handler-2 (window overlay pos)
  (message "help two")
  nil)
(overlay-put (make-overlay (point-min) (point-max)
                           (current-buffer) nil nil)
             'help-echo 'my-help-echo-handler-1)
(let ((overlay (make-overlay (point-min) (point-max)
                           (current-buffer) nil nil)))
  (overlay-put overlay 'help-echo 'my-help-echo-handler-2)
  (overlay-put overlay 'priority  10))
(dolist (overlay (overlays-in (point-min) (point-max)))
  (delete-overlay overlay))

;;------------------------------------------------------------------------------

(defun oeis-tooltip-help-echo-handler (window overlay pos)
  (let ((anum (with-current-buffer (window-buffer window)
                (save-excursion
                  (goto-char pos)
                  (oeis-anum-at-point)))))
    (message "%S" anum)
    (and anum
         ;; (concat "anum " anum)
         (concat "xx " (oeis-anum-to-name anum))
         )))

(oeis-anum-to-name "A000002")
;; A000002
;; A999999

(oeis-tooltip-help-echo-handler (selected-window)
                                (car (overlays-at (point)))
                                (point))
;; A000001

(overlay-get (car (overlays-at (point))) 'help-echo)
(progn
  (dolist (overlay (overlays-in (point-min) (point-max)))
    (delete-overlay overlay)
    '(if (eq 'oeis-tooltip-help-echo-handler (overlay-get overlay 'help-echo))
         ))

  (overlay-put (make-overlay (point-min) (point-max)
                             (current-buffer) nil nil)
               'help-echo "blah"
               ;; 'help-echo 'oeis-tooltip-help-echo-handler
               ))

(defun my-tooltip-help (event)
  (message "my-tooltip-help %S" event)
  nil)
(setq tooltip-functions '(tooltip-help-tips))
(setq tooltip-functions '(my-tooltip-help tooltip-help-tips))
(add-hook 'tooltip-functions 'my-tooltip-help)

;;------------------------------------------------------------------------------
catenary
echo
dog
(dictionary-tooltip-mode)

;;------------------------------------------------------------------------------
;; post-command-hook doesn't see up-to-date window-end

(defun my-post-command-handler ()
  (message "window-end %S" (window-end)))
(add-hook 'post-command-hook 'my-post-command-handler)

;;------------------------------------------------------------------------------

(defun oeis-tooltip-after-change-handler (beg end prev-len)
  (save-match-data
    (save-excursion
      ;; Extend to whole lines.
      ;;
      (goto-char beg)
      (setq beg (line-beginning-position))
      (goto-char end)
      (setq end (line-end-position))

      ;; Remove existing overlays.
      ;; emacs21 and xemacs21 don't have `remove-overlays' (new in emacs22),
      ;; but this is not the same anyway.  `remove-overlays' splits overlays
      ;; crossing the endpoints whereas here want to delete the lot.
      ;;
      (dolist (overlay (overlays-in beg end))
        (if (eq 'oeis-tooltip-help-echo-handler (overlay-get overlay 'help-echo))
            (delete-overlay overlay))))))

(defun oeis-tooltip-post-command-handler ()
  (if oeis-tooltip-mode
      (let ((case-fold-search nil)) ;; case sensitive
        (dolist (window (window-list))
          (with-selected-window window
            (window-start)
            (window-end)
            (with-current-buffer (window-buffer buffer)
              (goto-char beg)
              (while (re-search-forward "A[0-9]\\{6,7\\}\\|[ab][0-9]\\{6,7\\}.txt"
                                        end t)
                (overlay-put (make-overlay (match-beginning 0) (match-end 0)
                                           (current-buffer) nil nil)
                             'help-echo 'oeis-tooltip-help-echo-handler))))))))

(mapcar (lambda (window)
          (with-selected-window window
            (current-buffer)))
        (window-list))

;;------------------------------------------------------------------------------
;; ffap

;; ffap-string-at-point
;; 2*A000290
;; 2+A000290
;; 2,A000290
;; A000290(n)
;; A000290      squares
;; .emacs
;; elisp.info
;; info/elisp
;; pod-mode
;; RFC959
;; a000040.txt
;; 'b000290.txt'
;; b123456.txt
;; b999999.txt
;; b9999997.txt

;; a102419.txt   500 bytes
;; (apply 'buffer-substring (oeis-anum-at-point-bounds))

;;-----------------------------------------------------------------------------
;; old tooltip bits

  (if oeis-tooltip-mode
      (progn
        (if (eval-when-compile (fboundp 'make-local-hook))
            (make-local-hook 'after-change-functions)) ;; for xemacs21
        (add-hook 'after-change-functions
                  'oeis-tooltip-after-change
                  t   ;; append
                  t)) ;; buffer-local
    (remove-hook 'after-change-functions
                 'oeis-tooltip-after-change
                 t)) ;; buffer-local
  (oeis-tooltip-after-change (point-min) (point-max) 0)) ;; initial

(defun oeis-tooltip-after-change (beg end prev-len)
  (save-match-data
    (save-excursion
      ;; Extend to whole lines.
      ;;
      (goto-char beg)
      (setq beg (line-beginning-position))
      (goto-char end)
      (setq end (line-end-position))

      ;; Remove existing overlays.
      (dolist (overlay (overlays-in beg end))
        (if (eq 'oeis-tooltip-help-echo-handler (overlay-get overlay 'help-echo))
            (delete-overlay overlay)))

      (if oeis-tooltip-mode
          (let ((case-fold-search nil))
            (goto-char beg)
            (while (re-search-forward "\\b\\(A[0-9]\\{6,7\\}\\|[ab][0-9]\\{6,7\\}\\.txt\\)\\b"
                                      end t)
              (overlay-put (make-overlay (match-beginning 0) (match-end 0)
                                         (current-buffer) nil nil)
                           'help-echo 'oeis-tooltip-help-echo-handler)))))))

;;-----------------------------------------------------------------------------
(defun my-idle-timer (&rest args)
  (message "my-idle-timer runs ... %S" args))
(run-with-idle-timer 1
                     nil ;; no repeat
                     'my-idle-timer)
itimer-list
timer-list

;;-----------------------------------------------------------------------------
;; unused

(defun oeis-ellipsize (str)
  (if (< (length str) 40)
      str
    (concat (substring str 0 37) "...")))

(defun oeis-decimal-number-at-point (&optional separator)
  ;; checkdoc-params: (separator)
  "An internal part of oeis.el.
Return a decimal number at point.

If point is on a decimal number like 12.345 then return it as a
string \"12.345\".  If there's only 0 digits after the decimal
point then strip them."

  (unless separator (setq separator ","))
  (save-restriction
    (narrow-to-region (line-beginning-position) (line-end-position))
    (and (thing-at-point-looking-at "\\(\\<[0-9]*\\)?\\.[0-9]+\\b")
         (oeis--replace-regexp-in-string "\\.0+\\'" "" (match-string 0)))))


;;-----------------------------------------------------------------------------
;; junk


  ;; replace-regexp-in-string "%" "%25" str
  ;;                                       t   ;; fixedcase
  ;;                                       t)) ;; literal

;; &optional synchronous
;; (if synchronous
;;     ""
;;   )


;;-----------------------------------------------------------------------------
;; unused extract of A-number values to GP-DEFINE

(defun my-oeis-samples (anum)
  (interactive (list (progn (require 'oeis) (oeis-read-anum))))
  (save-excursion
    (beginning-of-line)
    (insert (concat comment-start " GP-DEFINE  " anum "_samples = {[\n"))
    (require 'lisp-mnt)
    (require 'oeis)
    (let ((str (or (my-oeis-stripped-line anum)
                   (error "A-number %s not found" anum))))
      (setq str (replace-regexp-in-string "," ", " str))
      (setq str (replace-regexp-in-string ",[ \t]*$" "" str))
      (save-restriction
        (narrow-to-region (point) (point))
        (let ((fill-prefix (concat comment-start " GP-DEFINE     "))
              (filladapt-mode nil))
          (insert (concat fill-prefix str "\n"))
          (fill-region (point-min) (point-max)))
        (goto-char (point-max))
        (insert (concat comment-start " GP-DEFINE  ]};\n"))))))
;; (my-oeis-samples "A000001")


;;------------------------------------------------------------------------------
;; JIT

(defun oeis-tooltip-jit-handler (window buffer pos)
  (message "oeis-tooltip-jit-handler %S %S %S" buffer pos (with-current-buffer buffer (point)))
  ;; Emacs  (WINDOW BUFFER POS)
  ;; XEmacs ...
  (with-current-buffer buffer
    (save-excursion
      (goto-char pos)
      (let* ((anum (oeis-anum-at-point))
             (name (and anum (oeis-anum-to-name anum))))
        (message "oeis-tooltip-jit-handler %S %S" 
                 (substring-no-properties anum) name)
        (and name
             (concat anum " " name))))))
(defun oeis-tooltip-jit (beg end)
  (save-excursion
    (goto-char beg)
    (while (re-search-forward
            "\\b\\(A[0-9]\\{6,7\\}\\|[ab][0-9]\\{6,7\\}\\.txt\\)\\b"
            end  ;; bound
            t)   ;; noerror
      ;; (message "oeis-tooltip-jit %S" (match-string 0))
      (let ((beg (match-beginning 0))
            (end (match-end 0)))
        (add-text-properties beg end
                             '(help-echo oeis-tooltip-jit-handler))))))
(require 'oeis)
(require 'oeis-tooltip)
(jit-lock-register 'oeis-tooltip-jit nil)
(font-lock-mode 1)
(describe-char (point))
;; A000001


;;; oeis-tooltip.el ends here

;;------------------------------------------------------------------------------
