;; Copyright 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (case-fold-search nil))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(areabrowse-mode))


    ;;-------------------------------------------------------------------------
    ;; local variables

    (with-temp-buffer
      (set (make-local-variable 'my-run-test-local-variable) 123)
      (unless (boundp 'my-run-test-local-variable)
        (error "oops, my-run-test-local-variable not bound"))

      (areabrowse-mode)

      (when (boundp 'my-run-test-local-variable)
        (error "my-run-test-local-variable still bound"))
      (setq count (1+ count))

      (unless (equal major-mode 'areabrowse-mode)
        (error "major-mode is %S" major-mode))
      (setq count (1+ count))

      (unless (equal mode-name "areabrowse")
        (error "mode-name is %S" mode-name))
      (setq count (1+ count))

      (unless (equal (current-local-map) areabrowse-mode-map)
        (error "(current-local-map) is not areabrowse-mode-map"))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; hooks run

    (let ((my-hook-count nil)
          (my-hook-func (lambda ()
                          (setq my-hook-count (1+ my-hook-count)))))

      ;; areabrowse-mode-hook
      (with-temp-buffer
        (add-hook 'areabrowse-mode-hook my-hook-func)
        (setq my-hook-count 0)
        (areabrowse-mode)
        (remove-hook 'areabrowse-mode-hook my-hook-func))
      (unless (equal my-hook-count 1)
        (error "Oops, areabrowse-mode-hook count %S" my-hook-count))
      (setq count (1+ count))


      ;; after-change-major-mode-hook, if available
      (if (not (boundp 'after-change-major-mode-hook))
          (message "no after-change-major-mode-hook in this emacs")
        (with-temp-buffer
          (add-hook 'after-change-major-mode-hook my-hook-func)
          (setq my-hook-count 0)
          (areabrowse-mode)
          (remove-hook 'after-change-major-mode-hook my-hook-func))
        (unless (equal my-hook-count 1)
          (error "Oops, after-change-major-mode-hook count %S" my-hook-count))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'areabrowse)
    (unload-feature 'areabrowse t)
    (require 'areabrowse)
    (unload-feature 'areabrowse t)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))

