;; Copyright 2009, 2010, 2012 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-predicate-false () nil)
(defun my-predicate-false2 () nil)

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (print-length nil)
        (tempbuf1 "*t-align-let--temp1*")
        (tempbuf2 "*t-align-let--temp2*"))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(texinfo-nobreak-enable))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((texinfo-mode-hook texinfo-nobreak-enable)))

    ;;-------------------------------------------------------------------------
    ;; texinfo-nobreak-enable

    (switch-to-buffer tempbuf1)

    (texinfo-nobreak-enable)
    (when (get 'fill-nobreak-predicate 'custom-type)
      (unless (memq 'texinfo-nobreak-p fill-nobreak-predicate)
        (error "texinfo-nobreak-p not in fill-nobreak-predicate")))
    (setq count (1+ count))

    (texinfo-nobreak-disable)
    (when (local-variable-p 'fill-nobreak-predicate (current-buffer))
      (error "disable initial leaves local fill-nobreak-predicate: %S"
             fill-nobreak-predicate))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; emacs21 predicate mangling

    (if (not (boundp 'fill-nobreak-predicate))
        (message "skip, no fill-nobreak-predicate (xemacs21)")

      (if (get 'fill-nobreak-predicate 'custom-type)
          (message "skip, fill-nobreak-predicate is a hook (emacs22)")

        ;;--------
        (set (make-local-variable 'fill-nobreak-predicate) 'my-predicate-false)
        (texinfo-nobreak-enable)
        (let ((got  fill-nobreak-predicate)
              (want '(lambda ()
                       (or (texinfo-nobreak-p)
                           (my-predicate-false)))))
          (unless (equal got want)
            (error "emacs21 enable single->lambda\ngot  %S\nwant %S" got want)))
        (setq count (1+ count))

        ;;--------
        (texinfo-nobreak-disable)
        (let ((got  fill-nobreak-predicate)
              (want '(lambda ()
                       (or (my-predicate-false)))))
          (unless (equal got want)
            (error "emacs21 disable lambda of one\ngot  %S\nwant %S" got want)))
        (setq count (1+ count))

        ;;--------
        (set (make-local-variable 'fill-nobreak-predicate)
             '(lambda ()
                (or (my-predicate-false2)
                    (my-predicate-false))))
        (texinfo-nobreak-enable)
        (let ((got fill-nobreak-predicate)
              (want '(lambda ()
                       (or (texinfo-nobreak-p)
                           (my-predicate-false2)
                           (my-predicate-false)))))
          (unless (equal got want)
            (error "emacs21 enable lambda2->lambda3\ngot  %S\nwant %S" got want)))
        (setq count (1+ count))

        ;;--------
        (texinfo-nobreak-disable)
        (let ((got  fill-nobreak-predicate)
              (want '(lambda ()
                       (or (my-predicate-false2)
                           (my-predicate-false)))))
          (unless (equal got want)
            (error "emacs21 disable lambda3->lambda2\ngot  %S\nwant %S" got want)))
        (setq count (1+ count))

        ;;--------
        (set (make-local-variable 'fill-nobreak-predicate)
             '(lambda ()
                (or (texinfo-nobreak-p))))
        (texinfo-nobreak-disable)
        (when (local-variable-p 'fill-nobreak-predicate (current-buffer))
          (error "disable of `or' leaves local fill-nobreak-predicate: %S"
                 fill-nobreak-predicate))
        (setq count (1+ count))
        ))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (switch-to-buffer tempbuf2)
      (unload-feature 'texinfo-nobreak t)

      (when (get 'fill-nobreak-predicate 'custom-type)
        (with-current-buffer tempbuf1
          (when (memq 'texinfo-nobreak-p fill-nobreak-predicate)
            (error "texinfo-nobreak-p still in tempbuf1")))
        (with-current-buffer tempbuf2
          (when (memq 'texinfo-nobreak-p fill-nobreak-predicate)
            (error "texinfo-nobreak-p still in tempbuf2"))))
      (setq count (1+ count)))

    ;; couple of cycles
    (require 'texinfo-nobreak)
    (unload-feature 'texinfo-nobreak t)
    (require 'texinfo-nobreak)
    (unload-feature 'texinfo-nobreak t)
    (setq count (1+ count))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'texinfo-nobreak))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))
