;; Copyright 2013, 2014 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-string-first-line (str)
  "Return the first line of STR.
The return doesn't include the terminating \"\\n\" (if any)."
  (substring str 0 (or (string-match "\n" str)
                       (length str))))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-upstr--temp*"))
    (switch-to-buffer tempbuf)
    (emacs-lisp-mode)

    (let ((st (copy-syntax-table (syntax-table))))
      (modify-syntax-entry ?$ "$$" st)
      (set-syntax-table st))


    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(upstr-up-list-or-string
              upstr-backward-up-list-or-string))

    ;; docstrings in autoload cookies
    (let ((autodoc-forward  (documentation 'upstr-up-list-or-string))
          (autodoc-backward (documentation 'upstr-backward-up-list-or-string)))
      (require 'upstr)
      (let ((doc-forward (my-string-first-line
                          (documentation 'upstr-up-list-or-string)))
            (doc-backward (my-string-first-line
                           (documentation 'upstr-backward-up-list-or-string))))

        (unless (equal autodoc-forward doc-forward)
          (error "Autoload docstring not same as `upstr-up-list-or-string'\nauto: %S\nfull: %S\n"
                 autodoc-forward doc-forward))
        (setq count (1+ count))

        (unless (equal autodoc-backward doc-backward)
          (error "Autoload docstring not same as `upstr-backward-up-list-or-string'"))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; upstr-up-list-or-string

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (pos  (nth 1 elem))
                    (arg  (nth 2 elem))
                    (want (nth 3 elem))
                    msg)
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))

                (condition-case err
                    (upstr-up-list-or-string arg)
                  (error
                   (setq msg (error-message-string err))))

                (let ((got (if msg
                               'error
                             (- (point) (point-min)))))
                  (unless (equal want got)
                    (goto-char (point-max))
                    (insert "\n")
                    (insert (format "arg=%s\n" arg))
                    (insert "---str---\n" str "\n")
                    (insert (make-string pos  ? )
                            (format "^--pos=%s\n" pos))

                    (if (numberp want)
                        (insert (make-string want ? )
                                (format "^--want=%s\n" want))
                      (insert (format "want=%S\n" want)))

                    (if (numberp got)
                        (insert (make-string got ? )
                                (format "^--got=%s\n" got))
                      (insert (format "got=%S\n" got)))

                    (when msg
                      (insert (format "\n%s\n" msg)))
                    (error "upstr-up-list-or-string wrong at %s\n"
                           count))))
              (setq count (1+ count)))

            '(("(a (b (c)))" 7 -1  6)
              ("(a (b (c)))" 7  1  9)

              ("\"(a (b (c)))\"" 8 -1  7)
              ("\"(a (b (c)))\"" 8 -2  4)
              ("\"(a (b (c)))\"" 8 -3  1)
              ("\"(a (b (c)))\"" 8 -4  0)
              ("\"(a (b (c)))\"" 8 -5  error)
              ("\"(a (b (c)))\"" 8  1  10)
              ("\"(a (b (c)))\"" 8  2  11)
              ("\"(a (b (c)))\"" 8  3  12)
              ("\"(a (b (c)))\"" 8  4  13)
              ("\"(a (b (c)))\"" 8  5  error)

              ("  \" unterminated string backward " 10 -1  2)
              ("  \" unterminated string forward "  10  1  error)

              (" $ blah $ " 5 -1 1)
              (" $ blah $ " 5  1 9)
              (" $ (blah) $ " 5 -1 3)
              (" $ (blah) $ " 5 -2 1)
              (" $ (blah) $ " 5 -3 error)
              (" $ (blah) $ " 5  1 9)
              (" $ (blah) $ " 5  2 11)
              (" $ (blah) $ " 5  3 error)

              (" [ $blah$ ] " 5 -1 3)
              (" [ $blah$ ] " 5 -2 1)
              (" [ $blah$ ] " 5 -3 error)
              (" [ $blah$ ] " 5  1 9)
              (" [ $blah$ ] " 5  2 11)
              (" [ $blah$ ] " 5  3 error)
              ))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'upstr)
    (unload-feature 'upstr t)
    (require 'upstr)
    (unload-feature 'upstr t)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
