;;; emacs21-arc-lzh-summarize.el --- archive-mode lzh summarize fix

;; Copyright 2016 Kevin Ryde

;; Copyright (C) 1995, 1997, 1998, 2001, 2002, 2003, 2004, 2005, 2006,
;;   2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.

;; This file was derived from GNU Emacs.

;; GNU Emacs is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.



;; This is the Emacs 22 `archive-lzh-summarize' extracted and massaged for
;; use in Emacs 21, to fix, umm, something which was wrong in the Emacs 21
;; version.  Use it with something like
;;
;;     (and (string-match "GNU Emacs 21" (emacs-version))
;;          (locate-library "emacs21-arc-lzh-summarize")
;;          (eval-after-load "arc-mode"
;;            '(require 'emacs21-arc-lzh-summarize)))


;; from emacs 22

(defun archive-lzh-summarize (&optional start)
  (let ((p (or start 1)) ;; 1 for .lzh, something further on for .exe
        (totalsize 0)
        (maxlen 8)
        files
        visual)
    (while (progn (goto-char p)		;beginning of a base header.
                  (looking-at "\\(.\\|\n\\)\\(.\\|\n\\)-l[hz][0-9ds]-"))
      (let* ((hsize   (char-after p))	;size of the base header (level 0 and 1)
             ;; Convert to float to avoid overflow for very large files.
             (csize   (archive-l-e (+ p 7) 4
                                   ;; 'float
)) ;size of a compressed file to follow (level 0 and 2),
					;size of extended headers + the compressed file to follow (level 1).
             (ucsize  (archive-l-e (+ p 11) 4
                                   ;; 'float
))	;size of an uncompressed file.
             (time1   (archive-l-e (+ p 15) 2))	;date/time (MSDOS format in level 0, 1 headers
             (time2   (archive-l-e (+ p 17) 2))	;and UNIX format in level 2 header.)
             (hdrlvl  (char-after (+ p 20))) ;header level
             thsize		;total header size (base + extensions)
             fnlen efnname osid fiddle ifnname width p2
             neh	;beginning of next extension header (level 1 and 2)
             mode modestr uid gid text dir prname
             gname uname modtime moddate)
        (if (= hdrlvl 3) (error "can't handle lzh level 3 header type"))
        (when (or (= hdrlvl 0) (= hdrlvl 1))
          (setq fnlen   (char-after (+ p 21))) ;filename length
          (setq efnname (let ((str (buffer-substring (+ p 22) (+ p 22 fnlen))))	;filename from offset 22
                          (if file-name-coding-system
                              (decode-coding-string str file-name-coding-system)
                            (string-as-multibyte str))))
          (setq p2      (+ p 22 fnlen))) ;
        (if (= hdrlvl 1)
            (setq neh (+ p2 3))         ;specific to level 1 header
          (if (= hdrlvl 2)
              (setq neh (+ p 24))))     ;specific to level 2 header
        (if neh		;if level 1 or 2 we expect extension headers to follow
            (let* ((ehsize (archive-l-e neh 2))	;size of the extension header
                   (etype (char-after (+ neh 2)))) ;extension type
              (while (not (= ehsize 0))
                (cond
                 ((= etype 1)	;file name
                  (let ((i (+ neh 3)))
                    (while (< i (+ neh ehsize))
                      (setq efnname (concat efnname (char-to-string (char-after i))))
                      (setq i (1+ i)))))
                 ((= etype 2)	;directory name
                  (let ((i (+ neh 3)))
                    (while (< i (+ neh ehsize))
                      (setq dir (concat dir
                                        (if (= (char-after i)
                                               255)
                                            "/"
                                          (char-to-string
                                           (char-after i)))))
                      (setq i (1+ i)))))
                 ((= etype 80)		;Unix file permission
                  (setq mode (archive-l-e (+ neh 3) 2)))
                 ((= etype 81)		;UNIX file group/user ID
                  (progn (setq uid (archive-l-e (+ neh 3) 2))
                         (setq gid (archive-l-e (+ neh 5) 2))))
                 ((= etype 82)		;UNIX file group name
                  (let ((i (+ neh 3)))
                    (while (< i (+ neh ehsize))
                      (setq gname (concat gname (char-to-string (char-after i))))
                      (setq i (1+ i)))))
                 ((= etype 83)		;UNIX file user name
                  (let ((i (+ neh 3)))
                    (while (< i (+ neh ehsize))
                      (setq uname (concat uname (char-to-string (char-after i))))
                      (setq i (1+ i)))))
                 )
                (setq neh (+ neh ehsize))
                (setq ehsize (archive-l-e neh 2))
                (setq etype (char-after (+ neh 2))))
              ;;get total header size for level 1 and 2 headers
              (setq thsize (- neh p))))
        (if (= hdrlvl 0)  ;total header size
            (setq thsize hsize))
        ;; OS ID field not present in level 0 header, use code 0 "generic"
        ;; in that case as per lha program header.c get_header()
        (setq osid (cond ((= hdrlvl 0)  0)
                         ((= hdrlvl 1)  (char-after (+ p 22 fnlen 2)))
                         ((= hdrlvl 2)  (char-after (+ p 23)))))
        ;; Filename fiddling must follow the lha program, otherwise the name
        ;; passed to "lha pq" etc won't match (which for an extract silently
        ;; results in no output).  As of version 1.14i it goes from the OS ID,
        ;; - For 'M' MSDOS: msdos_to_unix_filename() downcases always, and
        ;;   converts "\" to "/".
        ;; - For 0 generic: generic_to_unix_filename() downcases if there's
        ;;   no lower case already present, and converts "\" to "/".
        ;; - For 'm' MacOS: macos_to_unix_filename() changes "/" to ":" and
        ;;   ":" to "/"
        (setq fiddle (cond ((= ?M osid) t)
                           ((= 0 osid)  (string= efnname (upcase efnname)))))
        (setq ifnname (if fiddle (downcase efnname) efnname))
        (setq prname (if dir (concat dir ifnname) ifnname))
        (setq width (if prname (string-width prname) 0))
        (setq modestr (if mode (archive-int-to-mode mode) "??????????"))
        (setq moddate (if (= hdrlvl 2)
                          (archive-unixdate time1 time2) ;level 2 header in UNIX format
                        (archive-dosdate time2))) ;level 0 and 1 header in DOS format
        (setq modtime (if (= hdrlvl 2)
                          (archive-unixtime time1 time2)
                        (archive-dostime time1)))
        (setq text    (if archive-alternate-display
                          (format "  %8.0f  %5S  %5S  %s"
                                  ucsize
                                  (or uid "?")
                                  (or gid "?")
                                  ifnname)
                        (format "  %10s  %8.0f  %-11s  %-8s  %s"
                                modestr
                                ucsize
                                moddate
                                modtime
                                prname)))
        (setq maxlen (max maxlen width)
              totalsize (+ totalsize ucsize)
              visual (cons (vector text
                                   (- (length text) (length prname))
                                   (length text))
                           visual)
              files (cons (vector prname ifnname fiddle mode (1- p))
                          files))
        (cond ((= hdrlvl 1)
               ;; p needs to stay an integer, since we use it in goto-char
               ;; above.  Passing through `round' limits the compressed size
               ;; to most-positive-fixnum, but if the compressed size exceeds
               ;; that, we cannot visit the archive anyway.
               (setq p (+ p hsize 2 (round csize))))
              ((or (= hdrlvl 2) (= hdrlvl 0))
               (setq p (+ p thsize 2 (round csize)))))
        ))
    (goto-char (point-min))
    (set-buffer-multibyte default-enable-multibyte-characters)
    (let ((dash (concat (if archive-alternate-display
                            "- --------  -----  -----  "
                          "- ----------  --------  -----------  --------  ")
                        (make-string maxlen ?-)
                        "\n"))
          (header (if archive-alternate-display
                      "M   Length    Uid    Gid  File\n"
                    "M   Filemode    Length  Date         Time      File\n"))
          (sumline (if archive-alternate-display
                       "  %8.0f                %d file%s"
                     "              %8.0f                         %d file%s")))
      (insert header dash)
      (archive-summarize-files (nreverse visual))
      (insert dash
              (format sumline
                      totalsize
                      (length files)
                      (if (= 1 (length files)) "" "s"))
              "\n"))
    (apply 'vector (nreverse files))))

(defun archive-unixdate (low high)
  "Stringify Unix (LOW HIGH) date."
  (let ((str (current-time-string (cons high low))))
    (format "%s-%s-%s"
	    (substring str 8 10)
	    (substring str 4 7)
	    (substring str 20 24))))

(defun archive-unixtime (low high)
  "Stringify Unix (LOW HIGH) time."
  (let ((str (current-time-string (cons high low))))
    (substring str 11 19)))

(provide 'emacs21-arc-lzh-summarize.el)
